#pragma once

#define VAST_ENABLE_ARROW 0
#define VAST_ENABLE_ASAN 0
#define VAST_ENABLE_ASSERTIONS 1
#define VAST_ENABLE_BACKTRACE 1
#define VAST_ENABLE_BUILDID 1
#define VAST_ENABLE_BUNDLED_CAF 0
#define VAST_ENABLE_DEVELOPER_MODE 1
#define VAST_ENABLE_EXCEPTIONS 0
#define VAST_ENABLE_JEMALLOC 1
#define VAST_ENABLE_OPENSSL 1
#define VAST_ENABLE_RELOCATABLE_INSTALLATIONS 0
#define VAST_ENABLE_SDT 1
#define VAST_ENABLE_STATIC_EXECUTABLE 0
#define VAST_ENABLE_UBSAN 0
#define VAST_ENABLE_UNIT_TESTS 0

namespace vast::version {

/// Contains the full version string.
extern const char* version;

/// The individual version components.
extern unsigned short major;
extern unsigned short minor;
extern unsigned short patch;
extern unsigned short tweak;

/// Contains a hash of VAST's build tree.
extern const char* build_tree_hash;

} // namespace vast::version

#include <caf/config.hpp>

#define VAST_LOG_LEVEL_QUIET CAF_LOG_LEVEL_QUIET
#define VAST_LOG_LEVEL_CRITICAL 1
#define VAST_LOG_LEVEL_ERROR CAF_LOG_LEVEL_ERROR
#define VAST_LOG_LEVEL_WARNING CAF_LOG_LEVEL_WARNING
#define VAST_LOG_LEVEL_INFO CAF_LOG_LEVEL_INFO
#define VAST_LOG_LEVEL_VERBOSE 11
#define VAST_LOG_LEVEL_DEBUG CAF_LOG_LEVEL_DEBUG
#define VAST_LOG_LEVEL_TRACE CAF_LOG_LEVEL_TRACE

// clang-format off
// clang-format tries to seperate the @ with a space, hence we disable it here.
#define VAST_LOG_LEVEL VAST_LOG_LEVEL_DEBUG
// clang-format on

// TODO: Move everything below to a more appropriate place.

#if defined(CAF_CLANG)
#  define VAST_CLANG 1
#else
#  define VAST_CLANG 0
#endif

#if defined(CAF_GCC)
#  define VAST_GCC 1
#else
#  define VAST_GCC 0
#endif

#if VAST_CLANG
#  define VAST_DIAGNOSTIC_PUSH _Pragma("clang diagnostic push")
#  define VAST_DIAGNOSTIC_POP _Pragma("clang diagnostic pop")
#  define VAST_DIAGNOSTIC_IGNORE_DEPRECATED                                    \
    _Pragma("clang diagnostic ignored \"-Wdeprecated\"")
#  define VAST_DIAGNOSTIC_IGNORE_VLA_EXTENSION                                 \
    _Pragma("clang diagnostic ignored \"-Wvla-extension\"")                    \
      _Pragma("clang diagnostic ignored \"-Wgnu-folding-constant\"")
#  define VAST_DIAGNOSTIC_IGNORE_ADDRESS /* nop */
#elif VAST_GCC
#  define VAST_DIAGNOSTIC_PUSH _Pragma("GCC diagnostic push")
#  define VAST_DIAGNOSTIC_POP _Pragma("GCC diagnostic pop")
#  define VAST_DIAGNOSTIC_IGNORE_DEPRECATED                                    \
    _Pragma("GCC diagnostic ignored \"-Wdeprecated\"")
#  define VAST_DIAGNOSTIC_IGNORE_VLA_EXTENSION                                 \
    _Pragma("GCC diagnostic ignored \"-Wvla\"")
#  define VAST_DIAGNOSTIC_IGNORE_ADDRESS                                       \
    _Pragma("GCC diagnostic ignored \"-Waddress\"")
#endif

#ifdef CAF_BSD
#  define VAST_BSD 1
#else
#  define VAST_BSD 0
#endif
#ifdef CAF_LINUX
#  define VAST_LINUX 1
#else
#  define VAST_LINUX 0
#endif
#ifdef CAF_MACOS
#  define VAST_MACOS 1
#else
#  define VAST_MACOS 0
#endif
#ifdef CAF_WINDOWS
#  define VAST_WINDOWS 1
#else
#  define VAST_WINDOWS 0
#endif
#if !(VAST_BSD || VAST_LINUX || VAST_MACOS || VAST_WINDOWS)
#  error platform not supported
#endif
#ifdef CAF_POSIX
#  define VAST_POSIX 1
#else
#  define VAST_POSIX 0
#endif

// Convenience macros
#define VAST_IGNORE_UNUSED(x) CAF_IGNORE_UNUSED(x)
